package cz.vsb.kp;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.Fixture;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.World;

public class Bucket {

	private Sprite sprite;
	private Body body;
	public Fixture dnoFixture;
	private Texture img;
	private float centerX;
	private float centerY;
	private World world;

	public Bucket(Texture img, float centerX, float centerY, World world) {
		this.img = img;
		sprite = new Sprite(img);
		sprite.setOriginBasedPosition(img.getWidth() / 200f, img.getHeight() / 200f);
		sprite.setOrigin(0, 0);
		sprite.setSize(img.getWidth() / 100f, img.getHeight() / 100f);
		this.centerX = centerX - 1;
		this.centerY = centerY;
		this.world = world;
		createBody();
		setVelocity(10);
	}

	private void createBody() {
		int vyskaUchaKbeliku = 100;
		PolygonShape dno = new PolygonShape();
		dno.setAsBox(img.getWidth() / 200f, 0.02f, new Vector2(0, -img.getHeight() / 200f), 0);
		PolygonShape stenaPrava = new PolygonShape();
		stenaPrava.setAsBox(0.02f, (img.getHeight() - vyskaUchaKbeliku) / 200f,
				new Vector2(img.getWidth() / 200f, -vyskaUchaKbeliku / 200f), 0);
		PolygonShape stenaLeva = new PolygonShape();
		stenaLeva.setAsBox(0.02f, (img.getHeight() - vyskaUchaKbeliku) / 200f,
				new Vector2(-img.getWidth() / 200f, -vyskaUchaKbeliku / 200f), 0);
		PolygonShape vnitrek = new PolygonShape();
		vnitrek.setAsBox((img.getWidth() - 8) / 200f, ((img.getHeight() - vyskaUchaKbeliku) / 400f),
				new Vector2(0, -vyskaUchaKbeliku / 200f - (img.getHeight() - vyskaUchaKbeliku) / 400f + 0.04f), 0);
		FixtureDef detekceObsahu = new FixtureDef();
		detekceObsahu.isSensor = true;
		detekceObsahu.shape = vnitrek;
		BodyDef def = new BodyDef();
		def.type = BodyDef.BodyType.KinematicBody;
		body = world.createBody(def);
		dnoFixture = body.createFixture(dno, 0);
		body.createFixture(stenaPrava, 0);
		body.createFixture(stenaLeva, 0);
		body.createFixture(detekceObsahu);
		body.setTransform(centerX, centerY, 0);
		dno.dispose();
		stenaLeva.dispose();
		stenaPrava.dispose();
		vnitrek.dispose();
	}

	public float getX() {
		return body.getPosition().x;
	}

	public float getY() {
		return body.getPosition().y;
	}

	public float getWidth() {
		return sprite.getWidth();
	}

	public float getHeigh() {
		return sprite.getHeight();
	}

	public void setVelocity(float v) {
		if (v < 5 && v > -5) {
			body.setLinearVelocity(v, 0);
		}
		System.out.println(getVelocity());
	}

	public float getVelocity() {
		return body.getLinearVelocity().x;
	}

	public void draw(SpriteBatch batch) {
		sprite.setCenter(body.getPosition().x, body.getPosition().y);
		sprite.setOrigin(sprite.getWidth() / 2, sprite.getHeight() / 2);
		sprite.setRotation((float) (180 / Math.PI * body.getAngle()));
		sprite.draw(batch);
//		batch.draw(img, getX()-getWidth()/2, getY()-getHeigh()/2);
	}

	public void draw(ShapeRenderer shapeRenderer) {
//		Color c = new Color(0, 1, 0, 0.5f);
//		shapeRenderer.setColor(c);
//		shapeRenderer.rect(body.getPosition().x-getWidth()/2, body.getPosition().y - getHeigh()/2, getWidth(), getHeigh());
//		Color c1 = new Color(1, 1, 0, 0.5f);
//		shapeRenderer.setColor(c1);
//		shapeRenderer.rect(body.getPosition().x-getWidth()/2 + 10, body.getPosition().y - getHeigh()/2 + 10, getWidth() - 20, getHeigh() - 20);

	}

	public boolean isInside(Drop drop) {
		Rectangle bucket = new Rectangle(centerX - getWidth() / 2, centerY / getHeigh() / 2, getWidth(), getHeigh());
		Rectangle dropRect = new Rectangle(centerX, centerY, centerY, centerX);
		return bucket.overlaps(dropRect);
	}

}
