package cz.vsb.kp;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.Input.Buttons;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Matrix4;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Box2DDebugRenderer;
import com.badlogic.gdx.physics.box2d.World;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.ScreenUtils;
import com.badlogic.gdx.utils.TimeUtils;

public class GardenScreen implements Screen {

	private final DropGame dropGame;

	private float garderWidth = 8;
	private float garderHeight = 4.8f;
	
	private OrthographicCamera camera;
	private Texture bucketImg;
	private Texture dropImg;
	private Texture gardenImg;
	private Array<Drop> raindrops;
	private long lastDropTime;
	private Bucket bucket;
	
	static private boolean projectionMatrixSet;

	World world;
	Box2DDebugRenderer debugRenderer;

	public GardenScreen(final DropGame dropGame) {
		this.dropGame = dropGame;
		projectionMatrixSet = false;
		camera = new OrthographicCamera();
		camera.setToOrtho(false, garderWidth, garderHeight);
		world = new World(new Vector2(0, -10), true);
		world.setContactListener(new ContactDetector());
		debugRenderer = new Box2DDebugRenderer();

		bucketImg = new Texture(Gdx.files.internal("bucket.png"));
		dropImg = new Texture(Gdx.files.internal("drop.png"));
		gardenImg = new Texture(Gdx.files.internal("garden.png"));

		bucket = new Bucket(bucketImg, (int) (camera.viewportWidth / 2 - bucketImg.getWidth() / 200), 1.35f, world);
		dropGame.shapeRenderer.setProjectionMatrix(dropGame.batch.getProjectionMatrix());

		raindrops = new Array<>();
		spawnRaindrop();

	}

	@Override
	public void show() {
	}

	@Override
	public void render(float delta) {
		ScreenUtils.clear(1, 0, 0, 1);
		camera.update();
		dropGame.batch.setProjectionMatrix(camera.combined);
		myRender();
		debugRenderer.render(world, camera.combined);
		world.step(delta, 6, 2);
		if (TimeUtils.nanoTime() - lastDropTime > 1000000000) {
			spawnRaindrop();
		}
	}

	private int hp = 3;

	private void myRender() {
		dropGame.batch.begin();
		dropGame.batch.draw(gardenImg, 0, 0, camera.viewportWidth, camera.viewportHeight);
		for(int i=0; i<hp; i++) {
			dropGame.batch.draw(dropImg, 0+i*garderWidth/10*1.2f, garderWidth - garderWidth/10, garderWidth/10, garderWidth/10);
		}
		bucket.draw(dropGame.batch);
		for (Drop raindrop : raindrops) {
			raindrop.update();
			raindrop.draw(dropGame.batch);
		}
		Matrix4 originalMatrix = dropGame.batch.getProjectionMatrix().cpy(); // cpy() needed to properly set afterwards because calling set() seems to modify kept matrix, not replaces it
		dropGame.batch.setProjectionMatrix(originalMatrix.cpy().scale(1/120f, 1/120f, 1));
		dropGame.font.draw(dropGame.batch, "Body: " + 5, 10, 40);
		dropGame.batch.setProjectionMatrix(originalMatrix); //revert projection
		dropGame.batch.end();

		Gdx.gl.glEnable(GL20.GL_BLEND);
		Gdx.gl.glBlendFunc(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA);
		if (!projectionMatrixSet) {
			dropGame.shapeRenderer.setProjectionMatrix(dropGame.batch.getProjectionMatrix());
			projectionMatrixSet = true;
		}
//		Vector3 mouse = new Vector3(x, y, 0);
//		camera.unproject(mouse);
		dropGame.shapeRenderer.begin(ShapeType.Filled);
		bucket.draw(dropGame.shapeRenderer);
		dropGame.shapeRenderer.end();
		Gdx.gl.glDisable(GL20.GL_BLEND);

		if (Gdx.input.isKeyPressed(Input.Keys.LEFT)) {
			bucket.setVelocity(bucket.getVelocity()-1f);
		}
		if (Gdx.input.isKeyPressed(Input.Keys.RIGHT)) {
			bucket.setVelocity(bucket.getVelocity() + 1f);
		}
		if (!Gdx.input.isKeyPressed(Input.Keys.LEFT) && !Gdx.input.isKeyPressed(Input.Keys.RIGHT)) {
			bucket.setVelocity(bucket.getVelocity()*0.8f);
		}
		if (bucket.getX() < bucket.getWidth()/2 && bucket.getVelocity() < 0) {
			bucket.setVelocity(0);
		}
		if (bucket.getX() > 8 - bucket.getWidth()/2 && bucket.getVelocity() > 0) {
			bucket.setVelocity(0);
		}
		if (TimeUtils.nanoTime() - lastDropTime > 500000000) {
			spawnRaindrop();
		}
//		for (Iterator<Drop> iter = raindrops.iterator(); iter.hasNext();) {
//			Drop raindrop = iter.next();
//			raindrop.y -= 200 * Gdx.graphics.getDeltaTime();
//			if (raindrop.y + 161 < 0)
//				iter.remove();
//			if (raindrop.overlaps(bucket)) {
//				iter.remove();
//			}
//		}
		if (Gdx.input.isKeyJustPressed(Input.Keys.F)) {
			if (!Gdx.graphics.isFullscreen()) {
				Gdx.graphics.setFullscreenMode(Gdx.graphics.getDisplayMode());
			} else {
				Gdx.graphics.setWindowedMode(800, 600);
			}
		}
	}

	private void spawnRaindrop() {
		Drop raindrop = new Drop(dropImg, MathUtils.random(0, 800-6)/100f, 4.8f, world);
		raindrops.add(raindrop);
		lastDropTime = TimeUtils.nanoTime();
	}

	@Override
	public void resize(int width, int height) {
		/* nothing to do */
	}

	@Override
	public void pause() {
		/* nothing to do */
	}

	@Override
	public void resume() {
		/* nothing to do */
	}

	@Override
	public void hide() {
		/* nothing to do */
	}

	@Override
	public void dispose() {
		dropImg.dispose();
		bucketImg.dispose();
		gardenImg.dispose();
		world.dispose();
		debugRenderer.dispose();
	}

}
